@@ -153,6 +153,7 @@ static void
 decompress_gzip(int fd_in, int fd_out, const char *desc)
 {
 	char buffer[DPKG_BUFFER_SIZE];
+	int z_errnum;
 	gzFile gzfile = gzdopen(fd_in, "r");
 
 	if (gzfile == NULL)
@@ -163,7 +164,6 @@ decompress_gzip(int fd_in, int fd_out, const char *desc)
 
 		actualread = gzread(gzfile, buffer, sizeof(buffer));
 		if (actualread < 0) {
-			int z_errnum = 0;
 			const char *errmsg = gzerror(gzfile, &z_errnum);
 
 			if (z_errnum == Z_ERRNO)
@@ -179,6 +179,17 @@ decompress_gzip(int fd_in, int fd_out, const char *desc)
 			ohshite(_("%s: internal gzip write error"), desc);
 	}
 
+	z_errnum = gzclose(gzfile);
+	if (z_errnum) {
+		const char *errmsg;
+
+		if (z_errnum == Z_ERRNO)
+			errmsg = strerror(errno);
+		else
+			errmsg = zError(z_errnum);
+		ohshit(_("%s: internal gzip read error: %s"), desc, errmsg);
+	}
+
 	if (close(fd_out))
 		ohshite(_("%s: internal gzip write error"), desc);
 }
@@ -312,6 +323,8 @@ decompress_bzip2(int fd_in, int fd_out, const char *desc)
 			ohshite(_("%s: internal bzip2 write error"), desc);
 	}
 
+	BZ2_bzclose(bzfile);
+
 	if (close(fd_out))
 		ohshite(_("%s: internal bzip2 write error"), desc);
 }
@@ -782,7 +795,7 @@ decompress_zstd(int fd_in, int fd_out, const char *desc)
 	while ((just_read = fd_read(fd_in, buf_in, to_read))) {
 		ZSTD_inBuffer input = { buf_in, just_read, 0 };
 		while (input.pos < input.size) {
-			ssize_t actualwrite;
+			size_t actualwrite;
 			ZSTD_outBuffer output = { buf_out, buf_out_size, 0 };
 			to_read = ZSTD_decompressStream(dstream, &output , &input);
 			if (ZSTD_isError(to_read)) {
@@ -834,7 +847,7 @@ compress_zstd(int fd_in, int fd_out, struct compress_params *params, const char
 	while ((just_read = fd_read(fd_in, buf_in, to_read))) {
 		ZSTD_inBuffer input = { buf_in, just_read, 0 };
 		while (input.pos < input.size) {
-			ssize_t actualwrite;
+			size_t actualwrite;
 			ZSTD_outBuffer output = { buf_out, buf_out_size, 0 };
 			to_read = ZSTD_compressStream(cstream, &output , &input);
 			if (ZSTD_isError(to_read)) {
@@ -850,7 +863,7 @@ compress_zstd(int fd_in, int fd_out, struct compress_params *params, const char
 		}
 	}
 	do {
-		ssize_t actualwrite;
+		size_t actualwrite;
 		ZSTD_outBuffer output = { buf_out, buf_out_size, 0 };
 		end_res = ZSTD_endStream(cstream, &output);
 		if (ZSTD_isError(end_res)) {
