@@ -184,7 +184,7 @@ pkg_parse_verify(struct parsedb_state *ps,
   if ((ps->flags & pdb_recordavailable) ||
       pkg->status > PKG_STAT_HALFINSTALLED) {
     /* We always want usable architecture information (as long as the package
-     * is in such a state that it make sense), so that it can be used safely
+     * is in such a state that it makes sense), so that it can be used safely
      * on string comparisons and the like. */
     if (pkgbin->arch->type == DPKG_ARCH_NONE)
       parse_warn(ps, _("missing '%s' field"), "Architecture");
@@ -259,7 +259,7 @@ pkg_parse_verify(struct parsedb_state *ps,
                 _("package has status %s but no triggers pending"),
                 pkg_status_name(pkg));
 
-  /* FIXME: There was a bug that could make a not-installed package have
+  /* Note: There was a bug that could make a not-installed package have
    * conffiles, so we check for them here and remove them (rather than
    * calling it an error, which will do at some point). */
   if (!(ps->flags & pdb_recordavailable) &&
@@ -271,20 +271,17 @@ pkg_parse_verify(struct parsedb_state *ps,
     pkgbin->conffiles = NULL;
   }
 
-  /* XXX: Mark not-installed leftover packages for automatic removal on
-   * next database dump. This code can be removed after dpkg 1.16.x, when
-   * there's guarantee that no leftover is found on the status file on
-   * major distributions. */
+  /* Note: Mark not-installed leftover packages for automatic removal on
+   * next database dump. */
   if (!(ps->flags & pdb_recordavailable) &&
       pkg->status == PKG_STAT_NOTINSTALLED &&
       pkg->eflag == PKG_EFLAG_OK &&
       (pkg->want == PKG_WANT_PURGE ||
-       pkg->want == PKG_WANT_DEINSTALL ||
-       pkg->want == PKG_WANT_HOLD)) {
+       pkg->want == PKG_WANT_DEINSTALL)) {
     pkg_set_want(pkg, PKG_WANT_UNKNOWN);
   }
 
-  /* XXX: Mark not-installed non-arch-qualified selections for automatic
+  /* Note: Mark not-installed non-arch-qualified selections for automatic
    * removal, as they do not make sense in a multiarch enabled world, and
    * might cause those selections to be unreferencable from command-line
    * interfaces when there's other more specific selections. */
@@ -295,7 +292,7 @@ pkg_parse_verify(struct parsedb_state *ps,
       pkgbin->arch->type == DPKG_ARCH_EMPTY)
     pkg_set_want(pkg, PKG_WANT_UNKNOWN);
 
-  /* XXX: Versions before dpkg 1.13.10 did not blank the Origin and Bugs
+  /* Note: Versions before dpkg 1.13.10 did not blank the Origin and Bugs
    * fields, so there can be packages that should be garbage collected but
    * are lingering around. Blank them to make sure we will forget all about
    * them on the next database dump. */
