@@ -28,6 +28,7 @@
 use strict;
 use warnings;
 
+use List::Util qw(any none);
 use Cwd;
 use File::Basename;
 use File::Spec;
@@ -35,19 +36,18 @@ use File::Spec;
 use Dpkg ();
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
-use Dpkg::Util qw(:list);
-use Dpkg::Arch qw(debarch_eq debarch_is debarch_is_wildcard debarch_is_illegal);
+use Dpkg::Arch qw(:operators);
 use Dpkg::Deps;
 use Dpkg::Compression;
 use Dpkg::Conf;
 use Dpkg::Control::Info;
 use Dpkg::Control::Tests;
 use Dpkg::Control::Fields;
-use Dpkg::Index;
 use Dpkg::Substvars;
 use Dpkg::Version;
 use Dpkg::Vars;
 use Dpkg::Changelog::Parse;
+use Dpkg::Source::Format;
 use Dpkg::Source::Package qw(get_default_diff_ignore_regex
                              set_default_diff_ignore_regex
                              get_default_tar_ignore_pattern);
@@ -119,7 +119,7 @@ if (defined($options{opmode}) &&
     }
     if ($dir eq '.') {
 	# . is never correct, adjust automatically
-	$dir = basename(cwd());
+	$dir = basename(getcwd());
 	chdir '..' or syserr(g_("unable to chdir to '%s'"), '..');
     }
     # --format options are not allowed, they would take precedence
@@ -236,9 +236,29 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
     my $changelog = changelog_parse(%ch_options);
     my $control = Dpkg::Control::Info->new($controlfile);
 
-    my $srcpkg = Dpkg::Source::Package->new(options => \%options);
+    # <https://reproducible-builds.org/specs/source-date-epoch/>
+    $ENV{SOURCE_DATE_EPOCH} ||= $changelog->{timestamp} || time;
+
+    # Select the format to use
+    if (not defined $build_format) {
+        my $format_file = "$dir/debian/source/format";
+        if (-e $format_file) {
+            my $format = Dpkg::Source::Format->new(filename => $format_file);
+            $build_format = $format->get();
+        } else {
+            warning(g_('no source format specified in %s, ' .
+                       'see dpkg-source(1)'), 'debian/source/format')
+                if $options{opmode} eq 'build';
+            $build_format = '1.0';
+        }
+    }
+
+    my $srcpkg = Dpkg::Source::Package->new(format => $build_format,
+                                            options => \%options);
     my $fields = $srcpkg->{fields};
 
+    $srcpkg->parse_cmdline_options(@cmdline_options);
+
     my @sourcearch;
     my %archadded;
     my @binarypackages;
@@ -260,7 +280,7 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 	    my $dep;
 	    my $type = field_get_dep_type($_);
 	    $dep = deps_parse($v, build_dep => 1, union => $type eq 'union');
-	    error(g_('error occurred while parsing %s'), $_) unless defined $dep;
+	    error(g_('cannot parse %s field'), $_) unless defined $dep;
 	    my $facts = Dpkg::Deps::KnownFacts->new();
 	    $dep->simplify_deps($facts);
 	    $dep->sort() if $type eq 'union';
@@ -283,16 +303,9 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 
         my $pkg_summary = sprintf('%s %s %s %s', $p, $type, $sect, $prio);
 
-        $pkg_summary .= ' arch=' . join ',', split /\s+/, $arch;
+        $pkg_summary .= ' arch=' . join ',', split ' ', $arch;
 
         if (defined $profile) {
-            # If the string does not contain brackets then it is using the
-            # old syntax. Emit a fatal error.
-            if ($profile !~ m/^\s*<.*>\s*$/) {
-                error(g_('binary package stanza %s is using an obsolete ' .
-                         'Build-Profiles field syntax'), $p);
-            }
-
             # Instead of splitting twice and then joining twice, we just do
             # simple string replacements:
 
@@ -304,7 +317,9 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
             $profile =~ s/\s+/,/g;
             $pkg_summary .= " profile=$profile";
         }
-
+        if (defined $pkg->{'Protected'} and $pkg->{'Protected'} eq 'yes') {
+            $pkg_summary .= ' protected=yes';
+        }
         if (defined $pkg->{'Essential'} and $pkg->{'Essential'} eq 'yes') {
             $pkg_summary .= ' essential=yes';
         }
@@ -321,7 +336,8 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
                     push(@sourcearch, $v) unless $archadded{$v}++;
                 } else {
                     for my $a (split(/\s+/, $v)) {
-                        error(g_("'%s' is not a legal architecture string"), $a)
+                        error(g_("'%s' is not a legal architecture string " .
+                                 "in package '%s'"), $a, $p)
                             if debarch_is_illegal($a);
                         error(g_('architecture %s only allowed on its ' .
                                  "own (list for package %s is '%s')"),
@@ -330,7 +346,7 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
                         push(@sourcearch, $a) unless $archadded{$a}++;
                     }
                 }
-            } elsif (m/^Homepage$/) {
+            } elsif (m/^(?:Homepage|Description)$/) {
                 # Do not overwrite the same field from the source entry
             } else {
                 field_transfer_single($pkg, $fields);
@@ -363,8 +379,7 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
     $fields->{'Package-List'} = "\n" . join("\n", sort @pkglist);
 
     # Check if we have a testsuite, and handle manual and automatic values.
-    set_testsuite_field($fields);
-    set_testsuite_triggers_field($fields, @binarypackages);
+    set_testsuite_fields($fields, @binarypackages);
 
     # Scan fields of dpkg-parsechangelog
     foreach (keys %{$changelog}) {
@@ -393,29 +408,6 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 	$fields->{'Binary'} =~ s/(.{0,980}), ?/$1,\n/g;
     }
 
-    # Select the format to use
-    if (not defined $build_format) {
-	if (-e "$dir/debian/source/format") {
-	    open(my $format_fh, '<', "$dir/debian/source/format")
-	        or syserr(g_('cannot read %s'), "$dir/debian/source/format");
-	    $build_format = <$format_fh>;
-	    chomp($build_format) if defined $build_format;
-	    error(g_('%s is empty'), "$dir/debian/source/format")
-		unless defined $build_format and length $build_format;
-	    close($format_fh);
-	} else {
-	    warning(g_('no source format specified in %s, ' .
-	               'see dpkg-source(1)'), 'debian/source/format')
-		if $options{opmode} eq 'build';
-	    $build_format = '1.0';
-	}
-    }
-    $fields->{'Format'} = $build_format;
-    $srcpkg->upgrade_object_type(); # Fails if format is unsupported
-    # Parse command line options
-    $srcpkg->init_options();
-    $srcpkg->parse_cmdline_options(@cmdline_options);
-
     if ($options{opmode} eq 'print-format') {
 	print $fields->{'Format'} . "\n";
 	exit(0);
@@ -503,9 +495,9 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
     exit(0);
 }
 
-sub set_testsuite_field
+sub set_testsuite_fields
 {
-    my $fields = shift;
+    my ($fields, @binarypackages) = @_;
 
     my $testsuite_field = $fields->{'Testsuite'} // '';
     my %testsuite = map { $_ => 1 } split /\s*,\s*/, $testsuite_field;
@@ -513,6 +505,11 @@ sub set_testsuite_field
         error(g_('test control %s is not a regular file'),
               'debian/tests/control') unless -f _;
         $testsuite{autopkgtest} = 1;
+
+        my $tests = Dpkg::Control::Tests->new();
+        $tests->load("$dir/debian/tests/control");
+
+        set_testsuite_triggers_field($tests, $fields, @binarypackages);
     } elsif ($testsuite{autopkgtest}) {
         warning(g_('%s field contains value %s, but no tests control file %s'),
                 'Testsuite', 'autopkgtest', 'debian/tests/control');
@@ -523,27 +520,26 @@ sub set_testsuite_field
 
 sub set_testsuite_triggers_field
 {
-    my ($fields, @binarypackages) = @_;
+    my ($tests, $fields, @binarypackages) = @_;
     my %testdeps;
 
     # Never overwrite a manually defined field.
     return if $fields->{'Testsuite-Triggers'};
 
-    # We only support autopkgtests.
-    return unless -e "$dir/debian/tests/control";
-
-    my $tests = Dpkg::Control::Tests->new();
-    $tests->load("$dir/debian/tests/control");
-
     foreach my $test ($tests->get()) {
+        if (not exists $test->{Tests} and not exists $test->{'Test-Command'}) {
+            error(g_('test control %s is missing %s or %s field'),
+                  'debian/tests/control', 'Tests', 'Test-Command');
+        }
+
         next unless $test->{Depends};
 
         my $deps = deps_parse($test->{Depends}, use_arch => 0, tests_dep => 1);
         deps_iterate($deps, sub { $testdeps{$_[0]->{package}} = 1 });
     }
 
-    # Remove our own binaries and meta-depends.
-    foreach my $pkg (@binarypackages, qw(@ @builddeps@)) {
+    # Remove our own binaries and its meta-depends variant.
+    foreach my $pkg (@binarypackages, qw(@)) {
         delete $testdeps{$pkg};
     }
     $fields->{'Testsuite-Triggers'} = join ', ', sort keys %testdeps;
@@ -573,11 +569,10 @@ sub print_option {
 sub get_format_help {
     $build_format //= '1.0';
 
-    my $srcpkg = Dpkg::Source::Package->new();
-    $srcpkg->{fields}->{'Format'} = $build_format;
-    $srcpkg->upgrade_object_type(); # Fails if format is unsupported
+    my $srcpkg = Dpkg::Source::Package->new(format => $build_format);
 
     my @cmdline = $srcpkg->describe_cmdline_options();
+    return '' unless @cmdline;
 
     my $help_build = my $help_extract = '';
     my $help;
@@ -619,6 +614,8 @@ sub usage {
                            extract source package.
   -b, --build <dir>        build source package.
       --print-format <dir> print the format to be used for the source package.
+      --before-build <dir> run the corresponding source package format hook.
+      --after-build <dir>  run the corresponding source package format hook.
       --commit [<dir> [<patch-name>]]
                            store upstream changes in a new patch.')
     . "\n\n" . g_(
